@@ -1,6 +1,7 @@
 
 import datetime
 from typing import List,Callable
+import pytz
 import requests
 import hashlib
 import sys
@@ -11,7 +12,7 @@ from PIL import Image
 import numpy as np
 from queue import SimpleQueue
 import io
-
+from contextlib import contextmanager
 import time
 
 
@@ -123,7 +124,7 @@ class PytificationsMessageWithPhoto:
         if text != "": 
             request_data["message"] = text
         
-        Pytifications._add_to_message_pool(edit_message,request_data,self)
+        Pytifications._add_to_message_pool(edit_message,request_data,self,Pytifications._options)
         
         return True
 
@@ -182,7 +183,7 @@ class PytificationsMessage:
         
         
         
-        Pytifications._add_to_message_pool(edit_message,request_data,self)
+        Pytifications._add_to_message_pool(edit_message,request_data,self,Pytifications._options)
         
         return True
 
@@ -210,8 +211,56 @@ def datetime_diff(diff: datetime.timedelta):
     return datetime.datetime.fromtimestamp(diff.total_seconds()).strftime(format)
 
 class PytificationsOptions:
-    _start_time = datetime.datetime.now()
-    def __init__(self,send_current_time_on_message=False,send_app_run_time_on_message = False,script_alias = "") -> None:
+    _start_time = datetime.datetime.now(pytz.UTC)
+    _should_update_in_server = True
+
+    @staticmethod
+    @contextmanager
+    def with_custom_options(options):
+        """
+        Helper decorator to keep the options passed to this function while inside the if statement
+
+        More useful internally
+        """
+        current_options = Pytifications._options
+        with PytificationsOptions.no_update_in_server():
+            try:
+                Pytifications.set_options(options)
+                yield
+            finally:
+                Pytifications.set_options(current_options)
+        
+
+    @staticmethod
+    @contextmanager
+    def no_update_in_server():
+        """
+        INTERNAL
+        Helper decorator to avoid updating the alias of the script when inside the with statement
+        """
+        try:
+            PytificationsOptions._should_update_in_server = False
+            yield None
+        finally:
+            PytificationsOptions._should_update_in_server = True
+
+    @staticmethod
+    @contextmanager
+    def default_options():
+        """
+        Helper decorator to keep the default options while inside the with statement
+        """
+        current_options = Pytifications._options
+        
+        with PytificationsOptions.no_update_in_server() as c:
+            try:
+                Pytifications.set_options(PytificationsOptions())
+                yield None
+            finally:
+                Pytifications.set_options(current_options)
+
+
+    def __init__(self,send_current_time_on_message=False,send_app_run_time_on_message = False,script_alias = "",should_print_sent_messages=False) -> None:
         """
         Data class for the options in Pytifications
 
@@ -221,62 +270,84 @@ class PytificationsOptions:
             send_current_time_on_message: (:obj:`bool`) whether to send the current time on the bottom of messages sent and edited
             
             script_alias: (:obj:`str`) alias to use when sending the message. Any spaces will be replaced with an underscore. Will appear on the top of the messages as "Message sent from __alias_here__:"  and will be used to send commands to your script
+        
+            should_print_sent_messages: (:obj:`bool`) whether to print to console when a message is sent
         """
         
         self._send_app_run_time_on_message = send_app_run_time_on_message
         self._script_alias = script_alias.replace(" ","_")
         self._send_current_time_on_message = send_current_time_on_message
+        self._should_print_sent_messages = should_print_sent_messages
         
     
 
     def format_string(self,string):
+        """
+        Formats a string according to the options
+        """
         if self._send_app_run_time_on_message or self._send_current_time_on_message:
             string = f'{string}\n'
         if self._send_app_run_time_on_message:
-            string = f'{string}\nrun_time:\n{datetime_diff(datetime.datetime.now() - PytificationsOptions._start_time)}'
+            string = f'{string}\nrun_time:\n{datetime_diff(datetime.datetime.now(pytz.UTC) - PytificationsOptions._start_time)}'
         if self._send_current_time_on_message:
-            string = f'{string}\ncurrent_time:\n{datetime.datetime.now().strftime("%H:%M:%S")}'
-
+            string = f'{string}\ncurrent_time:\n{(datetime.datetime.now(pytz.UTC) + Pytifications._prefered_timezone).strftime("%H:%M:%S")}'
+        
         if self._script_alias != "":
             string = f'Message sent from "{self._script_alias}":\n\n{string}'
         
         return string
 
-def send_message(request_data,photo: Image,return_data: PytificationsMessage | PytificationsMessageWithPhoto):
-    try:
-        res = requests.post('https://pytifications.herokuapp.com/send_message',json=request_data)
-    except Exception as e:
-        print(f"Found error when sending message: {e}")
-        return False
-    
+def send_message(request_data,photo: Image,return_data: PytificationsMessage | PytificationsMessageWithPhoto,current_options=None):
+    if current_options == None:
+        current_options = Pytifications._options
+        send_message(request_data,photo,return_data,current_options)
+        return
+    with PytificationsOptions.with_custom_options(current_options):
+        try:
+            res = requests.post('https://pytifications.herokuapp.com/send_message',json=request_data)
+        except Exception as e:
+            print(f"Found error when sending message: {e}")
+            return False
+        
 
-    if res.status_code != 200:
-        print(f'could not send message. reason: {res.reason}')
-        return False
+        if res.status_code != 200:
+            print(f'could not send message. reason: {res.reason}')
+            return False
 
-    Pytifications._last_message_id = int(res.text)
+        Pytifications._last_message_id = int(res.text)
 
-    
-    return_data._message_id = int(res.text)
-    if photo != None:
-        print(f'sent message with photo: "{request_data["message"]}"')
-        return_data._image = photo
+        
+        return_data._message_id = int(res.text)
+        if photo != None:
+            if Pytifications._options._should_print_sent_messages:
+                print(f'sent message with photo: "{request_data["message"]}"')
+            return_data._image = photo
 
-    else:
-        print(f'sent message: "{request_data["message"]}"')
+        else:
+            if Pytifications._options._should_print_sent_messages:
+                print(f'sent message: "{request_data["message"]}"')
 
-def edit_message(request_data,return_data: PytificationsMessage | PytificationsMessageWithPhoto):
+def edit_message(request_data,return_data: PytificationsMessage | PytificationsMessageWithPhoto,current_options=None):
+    if current_options == None:
+        current_options = Pytifications._options
+        edit_message(request_data,return_data,current_options)
+        return
+    with PytificationsOptions.with_custom_options(current_options):
+        try:     
+            requests.patch('https://pytifications.herokuapp.com/edit_message',json=request_data)
+        except Exception as e:
+            print(f'Found exception while editing message: {e}')
+            return False
 
-    
-    try:     
-        requests.patch('https://pytifications.herokuapp.com/edit_message',json=request_data)
-    except Exception as e:
-        print(f'Found exception while editing message: {e}')
-        return False
-
-    print(f'edited message with id {return_data._message_id if not isinstance(return_data._message_id,InternalPytificationsQueuedTask) else Pytifications._last_message_id} to "{request_data["message"]}"')   
+        if Pytifications._options._should_print_sent_messages:
+            print(f'edited message with id {return_data._message_id if not isinstance(return_data._message_id,InternalPytificationsQueuedTask) else Pytifications._last_message_id} to "{request_data["message"]}"')   
 
 def send_registered_commands(extras):
+    """
+    INTERNAL
+
+    used to send the registered commands to the bot
+    """
     message = f'The following commands are registered for the script with alias "{Pytifications._options._script_alias}":'
 
     message += "\n\nDefault commands:"
@@ -298,24 +369,33 @@ def send_registered_commands(extras):
 
     options = Pytifications._options
     custom_options = PytificationsOptions()
-    Pytifications.set_options(custom_options)
-    Pytifications.send_message(message)
-    Pytifications.set_options(options)
+    with PytificationsOptions.no_update_in_server():
+        Pytifications.set_options(custom_options)
+        Pytifications.send_message(message)
+        Pytifications.set_options(options)
 
 
+def send_started_time(extras):
+    alias = Pytifications._options._script_alias
+    with PytificationsOptions.default_options():
+        Pytifications.send_message(f'script/application "{alias}" was started at:\n{(PytificationsOptions._start_time + Pytifications._prefered_timezone).strftime("%d/%m/%Y, %H:%M:%S")}')
+
 class Pytifications:
     _login = None
     _logged_in = False
+    _prefered_timezone = datetime.timedelta(hours=0)
     _password = None
     _loop = None
     _registered_callbacks = {
         "__set_message_id":{"function":lambda *args: Pytifications._add_to_message_pool(update_message_id,*args),"args":[]}
     }
     _registered_commands = {
-        "check_commands":{"function":send_registered_commands,"description":"shows the available commands for this script"}
+        "check_commands":{"function":send_registered_commands,"description":"shows the available commands for this script/application"},
+        "started_time":{"function":send_started_time,"description":"shows the time that the script/application was launched"}
     }
     _default_commands = [
-        "check_commands"
+        "check_commands",
+        "started_time"
     ]
     _commands_to_call_synchronous = []
     _message_pool: SimpleQueue[InternalPytificationsQueuedTask] = SimpleQueue()
@@ -329,6 +409,14 @@ class Pytifications:
 
     @staticmethod
     def _add_to_message_pool(function,*args):
+        """
+        INTERNAL
+        
+        Used to add a function to be run when the message pool is updated
+
+        Used to avoid stopping the application every time a message is sent
+        """
+        
         task = InternalPytificationsQueuedTask(function,[],args)
         Pytifications._message_pool.put(task)
 
@@ -368,7 +456,8 @@ class Pytifications:
             called_any = True
             callback["function"](*callback["args"])
         for command in Pytifications._commands_to_call_synchronous:
-            called_any = True
+            if command["command_name"] not in Pytifications._default_commands:
+                called_any = True
             command["function"](command["args"])
         Pytifications._commands_to_call_synchronous.clear()
         Pytifications._callbacks_to_call_synchronous.clear()
@@ -384,6 +473,13 @@ class Pytifications:
         
         for more information on the available options check :obj:`PytificationsOptions`
         """
+        if Pytifications._logged_in and PytificationsOptions._should_update_in_server and options._script_alias != Pytifications._options._script_alias:
+            requests.patch("https://pytifications.herokuapp.com/update_process_name",json={
+                "username":Pytifications._login,
+                "password_hash":hashlib.sha256(Pytifications._password.encode('utf-8')).hexdigest(),
+                "process_id":Pytifications._process_id,
+                "process_name":options._script_alias
+            })
         Pytifications._options = options
     
     @staticmethod
@@ -439,17 +535,25 @@ class Pytifications:
             print(f'could not login... reason: {res.text}')
             return False
         else:
+            json = res.json()
             Pytifications._logged_in = True
-            Pytifications._process_id = res.text
+            Pytifications._process_id = json['id']
+            Pytifications._prefered_timezone = datetime.timedelta(hours=json['prefered_timezone'])
+
             print(f'success logging in to pytifications! script id = {Pytifications._process_id}')
 
-        Thread(target=Pytifications._check_if_any_callbacks_to_be_called,daemon=True).start()
+        Thread(target=Pytifications._check_if_any_events_called,daemon=True).start()
         
         return True
 
     
     @staticmethod
-    def _check_if_any_callbacks_to_be_called():
+    def _check_if_any_events_called():
+        """
+        INTERNAL
+        
+        Used for the inside logic to check if any callbacks or commands to be called
+        """
         while True:
             time.sleep(0.5)
             if not Pytifications.am_i_logged_in():
@@ -489,11 +593,14 @@ class Pytifications:
                         if Pytifications._synchronous:
                                 Pytifications._commands_to_call_synchronous.append({
                                     "function":Pytifications._registered_commands[item['command']]["function"],
-                                    "args":item['args']
+                                    "args":item['args'],
+                                    "command_name":item["command"]
                                 })
                         else:
                             Pytifications._registered_commands[item['command']]["function"](item['args'])
-
+                    else:
+                        with PytificationsOptions.default_options():
+                            Pytifications.send_message(f'No command was registered with name "{item["command"]}".\n\nAre you sure this is the intended command?')
 
 
 
@@ -538,13 +645,18 @@ class Pytifications:
         if photo != None:
             request_data['photo'] = image_to_byte_array(photo)
 
-        task = Pytifications._add_to_message_pool(send_message,request_data,photo,return_data)
+        task = Pytifications._add_to_message_pool(send_message,request_data,photo,return_data,Pytifications._options)
         return_data._message_id = task
 
         return return_data
 
     @staticmethod
     def __edit_last_message(message_return: PytificationsMessage | PytificationsMessageWithPhoto,message:str = "",buttons: List[List[PytificationButton]] = []):
+        """
+        INTERNAL
+
+        method used to run the message editing in the event pool
+        """
         message = Pytifications._options.format_string(message)
         
 
@@ -565,7 +677,7 @@ class Pytifications:
         if message != "":
             request_data["message"] = message
 
-        task = Pytifications._add_to_message_pool(edit_message,request_data,message_return)
+        task = Pytifications._add_to_message_pool(edit_message,request_data,message_return,Pytifications._options)
         message_return._message_id = task
         
 
@@ -594,6 +706,11 @@ class Pytifications:
         
     @staticmethod
     def _check_login():
+        """
+        INTERNAL
+
+        Method that checks if already logged in inside this script
+        """
         if not Pytifications._logged_in:
             print('could not send pynotification, make sure you have called Pytifications.login("username","password")')
             return False
